package at.ac.uibk.gitsearch.service.criteria;

import java.io.Serializable;
import java.util.Objects;
import tech.jhipster.service.Criteria;
import tech.jhipster.service.filter.BooleanFilter;
import tech.jhipster.service.filter.Filter;
import tech.jhipster.service.filter.IntegerFilter;
import tech.jhipster.service.filter.LongFilter;
import tech.jhipster.service.filter.StringFilter;

/**
 * Criteria class for the {@link at.ac.uibk.gitsearch.domain.Statistics} entity. This class is used
 * in {@link at.ac.uibk.gitsearch.web.rest.StatisticsResource} to receive all the possible filtering options from
 * the Http GET request parameters.
 * For example the following could be a valid request:
 * {@code /statistics?id.greaterThan=5&attr1.contains=something&attr2.specified=false}
 * As Spring is unable to properly convert the types, unless specific {@link Filter} class are used, we need to use
 * fix type specific filters.
 */
@SuppressWarnings(
    {
        "PMD.GodClass",
        "PMD.AvoidFieldNameMatchingMethodName",
        "PMD.FieldDeclarationsShouldBeAtStartOfClass",
        "PMD.ShortMethodName",
        "PMD.ConfusingTernary",
    }
)
public class StatisticsCriteria implements Serializable, Criteria {

    private static final long serialVersionUID = 1L;

    private LongFilter id;

    private IntegerFilter views;

    private IntegerFilter downloads;

    private StringFilter exerciseID;

    private BooleanFilter badgeRewarded;

    private Boolean distinct;

    public StatisticsCriteria() {
        super();
    }

    public StatisticsCriteria(StatisticsCriteria other) {
        this.id = other.id == null ? null : other.id.copy();
        this.views = other.views == null ? null : other.views.copy();
        this.downloads = other.downloads == null ? null : other.downloads.copy();
        this.exerciseID = other.exerciseID == null ? null : other.exerciseID.copy();
        this.badgeRewarded = other.badgeRewarded == null ? null : other.badgeRewarded.copy();
        this.distinct = other.distinct;
    }

    @Override
    public StatisticsCriteria copy() {
        return new StatisticsCriteria(this);
    }

    public LongFilter getId() {
        return id;
    }

    public LongFilter id() {
        if (id == null) {
            id = new LongFilter();
        }
        return id;
    }

    public void setId(LongFilter id) {
        this.id = id;
    }

    public IntegerFilter getViews() {
        return views;
    }

    public IntegerFilter views() {
        if (views == null) {
            views = new IntegerFilter();
        }
        return views;
    }

    public void setViews(IntegerFilter views) {
        this.views = views;
    }

    public IntegerFilter getDownloads() {
        return downloads;
    }

    public IntegerFilter downloads() {
        if (downloads == null) {
            downloads = new IntegerFilter();
        }
        return downloads;
    }

    public void setDownloads(IntegerFilter downloads) {
        this.downloads = downloads;
    }

    public StringFilter getExerciseID() {
        return exerciseID;
    }

    public StringFilter exerciseID() {
        if (exerciseID == null) {
            exerciseID = new StringFilter();
        }
        return exerciseID;
    }

    public void setExerciseID(StringFilter exerciseID) {
        this.exerciseID = exerciseID;
    }

    public BooleanFilter getBadgeRewarded() {
        return badgeRewarded;
    }

    public BooleanFilter badgeRewarded() {
        if (badgeRewarded == null) {
            badgeRewarded = new BooleanFilter();
        }
        return badgeRewarded;
    }

    public void setBadgeRewarded(BooleanFilter badgeRewarded) {
        this.badgeRewarded = badgeRewarded;
    }

    public Boolean getDistinct() {
        return distinct;
    }

    public void setDistinct(Boolean distinct) {
        this.distinct = distinct;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        final StatisticsCriteria that = (StatisticsCriteria) o;
        return (
            Objects.equals(id, that.id) &&
            Objects.equals(views, that.views) &&
            Objects.equals(downloads, that.downloads) &&
            Objects.equals(exerciseID, that.exerciseID) &&
            Objects.equals(badgeRewarded, that.badgeRewarded) &&
            Objects.equals(distinct, that.distinct)
        );
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, views, downloads, exerciseID, badgeRewarded, distinct);
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "StatisticsCriteria{" +
            (id != null ? "id=" + id + ", " : "") +
            (views != null ? "views=" + views + ", " : "") +
            (downloads != null ? "downloads=" + downloads + ", " : "") +
            (exerciseID != null ? "exerciseID=" + exerciseID + ", " : "") +
            (badgeRewarded != null ? "badgeRewarded=" + badgeRewarded + ", " : "") +
            (distinct != null ? "distinct=" + distinct + ", " : "") +
            "}";
    }
}
