package at.ac.uibk.gitsearch.service.mapper;

import at.ac.uibk.gitsearch.domain.Review;
import at.ac.uibk.gitsearch.domain.ReviewHistoryEntry;
import at.ac.uibk.gitsearch.domain.ReviewRating;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.repository.jpa.ReviewRatingRepository;
import at.ac.uibk.gitsearch.repository.jpa.ReviewRepository;
import at.ac.uibk.gitsearch.repository.jpa.UserRepository;
import at.ac.uibk.gitsearch.service.dto.ReviewHistoryEntryDTO;
import liquibase.pro.packaged.D;
import liquibase.pro.packaged.E;
import org.mapstruct.BeanMapping;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Named;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Mapper for the entity {@link ReviewHistoryEntry} and its DTO
 * {@link ReviewHistoryEntryDTO}.
 *
 * @apiNote: Not derived from EntityMapper, because we need the context
 */
@SuppressWarnings({ "PMD.AvoidDuplicateLiterals", "PMD.TooManyMethods" })
@Mapper(componentModel = "spring", uses = {})
@Service
public abstract class ReviewHistoryEntryMapper implements EntityMapper<ReviewHistoryEntryDTO, ReviewHistoryEntry> {

    @Autowired
    private ReviewRepository reviewRepository;

    @Autowired
    private ReviewRatingRepository reviewRatingRepository;

    @Autowired
    private UserRepository userRepository;

    @Override
    @Mapping(target = "review", source = "review", qualifiedByName = "mapReview2Long")
    @Mapping(target = "user", source = "user", qualifiedByName = "mapUser2Long")
    @Mapping(target = "reviewRating", source = "reviewRating", qualifiedByName = "mapReviewRating2Long")
    public abstract ReviewHistoryEntryDTO toDto(ReviewHistoryEntry s);

    @Override
    @Mapping(target = "review", source = "review", qualifiedByName = "mapReview")
    @Mapping(target = "reviewRating", source = "reviewRating", qualifiedByName = "mapReviewRating")
    @Mapping(target = "user", source = "user", qualifiedByName = "mapUser")
    public abstract ReviewHistoryEntry toEntity(ReviewHistoryEntryDTO dto);

    @Override
    @Named("partialUpdate")
    @BeanMapping(nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE)
    @Mapping(target = "review", source = "review", qualifiedByName = "mapReview")
    @Mapping(target = "reviewRating", source = "reviewRating", qualifiedByName = "mapReviewRating")
    @Mapping(target = "user", source = "user", qualifiedByName = "mapUser")
    public abstract void partialUpdate(@MappingTarget ReviewHistoryEntry entity, ReviewHistoryEntryDTO dto);

    @Named("partialUpdate")
    @BeanMapping(nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE)
    public abstract void partialUpdate(@MappingTarget E entity, D dto);

    @Named("mapReview")
    public Review mapReview(Long valueLong) {
        if (valueLong == null) {
            return null;
        }
        return (reviewRepository.getById(valueLong));
    }

    @Named("mapReviewRating")
    public ReviewRating mapReviewRating(Long valueLong) {
        if (valueLong == null) {
            return null;
        }
        return reviewRatingRepository.getById(valueLong);
    }

    @Named("mapReview2Long")
    public Long map(Review value) {
        if (value == null) {
            return null;
        }
        return value.getId();
    }

    @Named("mapReviewRating2Long")
    public Long map(ReviewRating value) {
        if (value == null) {
            return null;
        }
        return value.getId();
    }

    @Named("mapUser")
    public User mapUser(Long valueLong) {
        if (valueLong == null) {
            return null;
        }
        return (userRepository.getById(valueLong));
    }

    @Named("mapUser2Long")
    public Long map(User value) {
        if (value == null) {
            return null;
        }
        return value.getId();
    }
}
