import { Component, Input, Output, EventEmitter, AfterViewChecked } from '@angular/core';
import { SearchService } from 'app/search/service/search-service';
import { WatchlistManager } from 'app/shared/watchlist/watchlist-manager';
import { ExerciseService } from '../service/exercise.service';
import { ExtendedSearchResultDTO, hasChildren, SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';

@Component({
  selector: 'jhi-exercise-card',
  templateUrl: './exercise-card.component.html',
  styleUrls: ['./exercise-card.component.scss'],
})
export class ExerciseCardComponent implements AfterViewChecked {
  @Input() autoOpenDetails = false; // if true opens details dialog automatically

  @Output() exerciseSelectionEvent = new EventEmitter<ExtendedSearchResultDTO>();

  @Input() get referencedExercise(): SearchResultDTO | undefined {
    return this.exercise;
  }
  set referencedExercise(exercise: SearchResultDTO | undefined) {
    this.exercise = exercise as ExtendedSearchResultDTO;
  }

  exercise: ExtendedSearchResultDTO | undefined;

  constructor(
    protected searchService: SearchService,
    private watchlistManager: WatchlistManager,
    private exerciseService: ExerciseService
  ) {}

  ngAfterViewChecked(): void {
    if (this.autoOpenDetails) {
      const button = document.querySelector('.exerciseDetailButton');
      if (button instanceof HTMLElement) {
        button.click();
        this.autoOpenDetails = false;
      }
    }
  }

  selectExercise(): void {
    this.exercise = this.exerciseService.populateExerciseWithData(this.exercise!);
    this.exerciseSelectionEvent.emit(this.exercise);
  }

  /**
   * correct missing image urls
   */
  correctImageURL(event: Event): void {
    const img = event.srcElement as HTMLImageElement;
    if (img) {
      img.src = '/content/images/Logo_codeAbility_4c_300dpi_RGB3.gif';
    }
  }

  hasChildren(): boolean {
    return hasChildren(this.exercise);
  }
}
